﻿//////////////////////////////////////////////
// BoundingBox.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Frustum ;
	class Node ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkMaths
#include <NilkinsMaths/Algebra/Vector.h>

// Standards
#include <array>

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT BoundingBox final
	{
		private :
		
			// Attributs
			// En interne, nos 8 points
			std::array<nkMaths::Vector, 8> _points ;

			// Les 8 points transformés après application node
			std::array<nkMaths::Vector, 8> _transformedPoints ;

		public :

			// Constructeur, destructeur
			BoundingBox () ;
			BoundingBox (const nkMaths::Vector& center, float cubeHalfSide) ;
			BoundingBox (const nkMaths::Vector& center, const nkMaths::Vector& axisAlignedHalfSides) ;
			~BoundingBox () ;

			// Getters
			const std::array<nkMaths::Vector, 8>& getPoints () const ;
			const std::array<nkMaths::Vector, 8>& getTransformedPoints () const ;
			nkMaths::Vector getCenter () const ;
			nkMaths::Vector getAxisAlignedSides () const ;

			// Setters
			void setPoints (const std::array<nkMaths::Vector, 8>& points) ;

			// Updates
			void updateTransformation (Node* node) ;

			// Checks
			bool checkAgainst (const Frustum& frustum) const ;
	} ;
}