﻿//////////////////////////////////////////////
// Mesh.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class BoundingBox ;

	struct VertexComponent ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

#include "../Resources/Resource.h"

#include "VertexComposition.h"

// nkExport
#include <NilkinsExport/Exportable/Exportable.h>

/// Class ------------------------------------
	
namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT Mesh : public Resource, public nkExport::Exportable
	{
		protected :
		
			// Attributs
			// Les décomptes
			unsigned int _vertexCount ;
			unsigned int _indexCount ;

			// Bounds
			BoundingBox* _bounds ;
			// Id
			unsigned int _id ;

			// La composition de ce mesh
			VertexComposition _composition ;

			// Le pointeur sur le buffer de points CPU
			VertexComponent* _vertexCpuArray ;
			float* _vertexCpuFinalArray ;
			unsigned int* _indexCpuArray ;

			// Si le packed est déjà prêt ou pas
			bool _packedBufferReady ;
			// Calcul auto ou pas des bounds demandé
			bool _autoComputeBounds ;
		
		public :
		
			// Constructeur, destructeur
			Mesh () ;
			virtual ~Mesh () ;

			// Chargement, déchargement
			void resetCpuBuffers () ;

			// Init
			bool fillPackedFromUnpacked () ;
			void computeBounds () ;

			// Getter
			virtual int getVertexBufferCount () const ;
			virtual unsigned int getVertexCount () const ;
			virtual unsigned int getIndexCount () const ;
			unsigned int getId () const ;
			bool getPackedBufferReady () const ;
			bool getAutoComputeBounds () const ;
			BoundingBox* getBounds () const ;

			// Setters
			void setVertexCount (int value) ;
			void setIndexCount (int value) ;
			void setId (unsigned int value) ;
			void setPackedBufferReady (bool value) ;
			void setAutoComputeBounds (bool value) ;

			// Buffer access : Vertex unpacked
			VertexComponent* getCurrentVertexArray () const ;
			VertexComponent* getNewVertexArray (unsigned int requestedPointNumber) ;

			// Buffer access : Vertex packed
			float* getCurrentVertexPackedArray () const ;
			float* getNewVertexPackedArray (unsigned int requestedPointNumber) ;

			// Buffer access : Index
			unsigned int* getCurrentIndexArray () const ;
			unsigned int* getNewIndexArray (unsigned int requestedIndexNumber) ;

			// Composition vertex
			void setVertexComposition (const VertexComposition& compo) ;
			VertexComposition& getCurrentVertexComposition () ;

			// Préparations pour d'autres usages
			virtual void prepareForGeometryStream () = 0 ;

			// Default loading
			virtual bool load () override ;

			// Libération mémoire
			virtual void freeShadowBuffers (bool cleanUnpacked, bool cleanPacked) ;

			// Data generation
			void generateTangent () ;
			void generateBinormal () ;

			// Import / Export
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}