﻿//////////////////////////////////////////////
// RelativeLayout.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkWinUi
#include "../Dll/DllDefines.h"

#include "Layout.h"

// nkMaths
#include <NilkinsMaths/Geometry/Rectangle.h>

// Standards
#include <vector>

/// Internals --------------------------------

namespace nkWinUi
{
	enum class RELATIVE_POSITION
	{
		UNKNOWN = 0,
		LEFT,
		RIGHT,
		OVER,
		UNDER,
		OVER_LEFT,
		OVER_RIGHT,
		UNDER_LEFT,
		UNDER_RIGHT
	} ;

	struct RelativeLayoutEntryInfo
	{
		Component* _component ;
		RELATIVE_POSITION _position ;
		nkMaths::Rectangle  _area ;

		RelativeLayoutEntryInfo* _relativeTo ;
		std::vector<RelativeLayoutEntryInfo*> _children ;
	} ;
}

/// Class ------------------------------------
	
namespace nkWinUi
{
	class DLL_WINUI_EXPORT RelativeLayout final : public Layout
	{
		private :

			// Nos components à maintenir en vie
			std::vector<RelativeLayoutEntryInfo*> _entries ;
			// Utilisé sous forme de tree, on keep les roots ici
			std::vector<RelativeLayoutEntryInfo*> _roots ;
		
		public :
		
			// Constructeur, destructeur
			RelativeLayout () ;
			~RelativeLayout () ;

			// Getters
			Component* getComponentAt (unsigned int index) ;

			// Setters
			virtual void setParentComponent (Component* parent) override ;

			// Components
			void addComponent (Component* component, const nkMaths::Rectangle& area) ;
			void addComponent (Component* component, const nkMaths::Rectangle& area, RELATIVE_POSITION position, Component* relativeTo) ;
			virtual void removeComponent (Component* component) override ;

			// Update
			virtual void update () override ;

			// Import / Export
			virtual void exportComponentsNodeEntry (nkExport::Node* rootNode) override ;
			virtual void processComponentsNodeEntry (nkExport::Node* rootNode) override ;
	} ;
}